;Version 1.01 FINAL.
;Fixed some bugs in coldet.bb file
;Added the ability to create empty coldet models
; and add manually tris to them.


;------------------------------------------------------------------------
;COLLISION POINT BANK
Global col_point=CreateBank(3*4)

;COLLIDING TRIANGLES BANK
Global col_verts1=CreateBank(9*4)
Global col_verts2=CreateBank(9*4)

;COLLIDING TRIANGLES Index BANK
Global col_tri1=CreateBank(1*4)
Global col_tri2=CreateBank(1*4)


;RAY COLLISION BANKS
Global ray_origin=CreateBank(3*4)
Global ray_direction=CreateBank(3*4)

;sphere centers
Global col_center1=CreateBank(3*4)
Global col_center2=CreateBank(3*4)


;matrix
Global col_dummy_pivot
Global col_matrix = CreateBank(16*4)

;arrays for nx,ny,nz calculations
Dim cnrx1#(0),cnry1#(0),cnrz1#(0)
Dim cnrx2#(0),cnry2#(0),cnrz2#(0)
;------------------------------------------------------------------------



;FUNCTIONS---------------------------------------------------------------





;------------------------------------------------------------------------
;Start
;------------------------------------------------------------------------
Function coldet_start()

	col_dummy_pivot = CreatePivot()
	PokeFloat col_matrix, 12,0
	PokeFloat col_matrix, 28,0
	PokeFloat col_matrix, 44,0
	PokeFloat col_matrix, 60,1

Dim cnrx1#(1),cnry1#(1),cnrz1#(1)
Dim cnrx2#(1),cnry2#(1),cnrz2#(1)

End Function


;------------------------------------------------------------------------
;End
;------------------------------------------------------------------------
Function coldet_end()

	FreeEntity col_dummy_pivot
	FreeBank col_point
	FreeBank col_verts1
	FreeBank col_verts2
	FreeBank col_tri1
	FreeBank col_tri2
	FreeBank ray_origin
	FreeBank ray_direction
	FreeBank col_center1
	FreeBank col_center2
	FreeBank col_matrix

Dim cnrx1#(0),cnry1#(0),cnrz1#(0)
Dim cnrx2#(0),cnry2#(0),cnrz2#(0)

End Function






;------------------------------------------------------------------------
;This function creates a coldet compatible model.
;By this we mean that we use the global vertex positions of the model,
;since coldet does not work with scales.

;dynamic=0 the model is static.
;dyanmic=1 the model moves and/or rotates.
;------------------------------------------------------------------------

Function coldet_make(entity,dynamic)

Local model
Local cbank,count%
Local surf,tris,verts
Local v0%,v1%,v2%,point%
Local vx#,vy#,vz#
Local rx#,ry#,rz#
Local px#,py#,pz#


Local surfs=CountSurfaces(entity)
If surfs=0 Return 0

;We want a 0 rotated and 0 positioned object object
;We will return it to it's original state after the coldet model is created
rx=EntityPitch(entity,1)
ry=EntityYaw(entity,1)
rz=EntityRoll(entity,1)
RotateEntity entity,0,0,0,1
px=EntityX(entity,1)
py=EntityY(entity,1)
pz=EntityZ(entity,1)
PositionEntity entity,0,0,0,1

For i=1 To surfs
surf=GetSurface(entity,i)
tris=tris+CountTriangles(surf)
Next


;make the coldet model with triangles=tris and  dynamic= 0 or 1

;NOTE: If you not asign the tris number the coldet model
;      will still be constructed. The tris no. assignment 
;      is for faster construction speed of the coldet model.

If dynamic>1 dynamic=1
If dynamic<0 dynamic=0

model=coldet_make_model(tris,dynamic)

cbank=CreateBank(tris*9*4)


count=0

For i=1 To surfs
surf=GetSurface(entity,i)

	For k=0 To CountTriangles(surf)-1
	
		v0=TriangleVertex(surf,k,0)
		v1=TriangleVertex(surf,k,1)
		v2=TriangleVertex(surf,k,2)

			vx=VertexX(surf,v0)
			vy=VertexY(surf,v0)
			vz=VertexZ(surf,v0)
			TFormPoint vx,vy,vz,entity,0
			PokeFloat cbank,count+0,TFormedX()
			PokeFloat cbank,count+4,TFormedY()
			PokeFloat cbank,count+8,TFormedZ()
		
			vx=VertexX(surf,v1)
			vy=VertexY(surf,v1)
			vz=VertexZ(surf,v1)
			TFormPoint vx,vy,vz,entity,0
			PokeFloat cbank,count+12,TFormedX()
			PokeFloat cbank,count+16,TFormedY()
			PokeFloat cbank,count+20,TFormedZ()

			vx=VertexX(surf,v2)
			vy=VertexY(surf,v2)
			vz=VertexZ(surf,v2)
			TFormPoint vx,vy,vz,entity,0
			PokeFloat cbank,count+24,TFormedX()
			PokeFloat cbank,count+28,TFormedY()
			PokeFloat cbank,count+32,TFormedZ()
			
		count=count+36
	
	Next
	
Next
			
			
coldet_add_model(model,cbank,tris*9)

FreeBank cbank

coldet_finalize(model)


;set it back to it's original rotation and position
RotateEntity entity,rx,ry,rz,1
PositionEntity entity,px,py,pz,1

coldet_setmatrix(entity,model)



Return model

End Function






;------------------------------------------------------------------------
;USE THIS ONLY IF YOU CREATE A COLDET MODEL MANUALLY
;This function just makes an "empty" coldet model and specifies the number of tris
;Use this function against the "coldet_make" function if you want
;to add manually triangles to a coldet model.

;NOTE: If you not asign the tris number the coldet model
;      will still be constructed. The tris no. assignment 
;      is for faster construction speed of the coldet model.
;------------------------------------------------------------------------

Function coldet_model_start(tris=0,dynamic=0)
If dynamic>1 dynamic=1
If dynamic<0 dynamic=0
Return coldet_make_model(tris,dynamic)
End Function




;------------------------------------------------------------------------
;USE THIS ONLY IF YOU CREATE A COLDET MODEL MANUALLY
;Add manually a triangle to the coldet model.
;We pass here also the blitz model to get the global vertex positions.
;We rotate the blitz model to 0,0,0 and back to it's original rotation
;because it could be already rotated.
;------------------------------------------------------------------------

Function coldet_add_tri(b_ent,c_ent,vx1#,vy1#,vz1#,vx2#,vy2#,vz2#,vx3#,vy3#,vz3#)

Local tvx1#,tvy1#,tvz1#
Local tvx2#,tvy2#,tvz2#
Local tvx3#,tvy3#,tvz3#
Local px#,py#,pz#
Local rx#,ry#,rz#



rx=EntityPitch(b_ent,1)
ry=EntityYaw(b_ent,1)
rz=EntityRoll(b_ent,1)
px=EntityX(b_ent,1)
py=EntityY(b_ent,1)
pz=EntityZ(b_ent,1)

;We want a 0 rotated and 0 positioned object object
;We will return it to it's original state after the coldet tri is created
RotateEntity b_ent,0,0,0,1
PositionEntity b_ent,0,0,0,1

TFormPoint vx1,vy1,vz1,b_ent,0
tvx1=TFormedX()
tvy1=TFormedY()
tvz1=TFormedZ()
TFormPoint vx2,vy2,vz2,b_ent,0
tvx2=TFormedX()
tvy2=TFormedY()
tvz2=TFormedZ()
TFormPoint vx3,vy3,vz3,b_ent,0
tvx3=TFormedX()
tvy3=TFormedY()
tvz3=TFormedZ()

coldet_addtri(c_ent,tvx1,tvy1,tvz1,tvx2,tvy2,tvz2,tvx3,tvy3,tvz3)

;set it back to it's original rotation and position
RotateEntity b_ent,rx,ry,rz,1
PositionEntity b_ent,px,py,pz,1
End Function 


;------------------------------------------------------------------------
;USE THIS ONLY IF YOU CREATE A COLDET MODEL MANUALLY
;Finalize the coldet model and set it's matrix
;------------------------------------------------------------------------

Function coldet_model_end(b_ent,c_ent)
	coldet_finalize(c_ent)
	coldet_setmatrix(b_ent,c_ent)
End Function






;------------------------------------------------------------------------
;This function sets the transformation matrix of the coldet model.
;It is neccesary to call it only when moving and rotating
;a dynamic object.
;For Static objects you only need to call it once.

;b_ent=the blitz entity
;c_ent=the coldet entity pointer !
;------------------------------------------------------------------------

Function coldet_setmatrix(b_ent,c_ent)

;we use a dummy pivot to get pitch,yaw,roll because coldet doesn't work with scale values
RotateEntity col_dummy_pivot,EntityPitch(b_ent,1),EntityYaw(b_ent,1),EntityRoll(b_ent,1)

	PokeFloat col_matrix, 0,GetMatElement(col_dummy_pivot,0,0)
	PokeFloat col_matrix, 4,GetMatElement(col_dummy_pivot,0,1)
	PokeFloat col_matrix, 8,GetMatElement(col_dummy_pivot,0,2)

	PokeFloat col_matrix, 16,GetMatElement(col_dummy_pivot,1,0)
	PokeFloat col_matrix, 20,GetMatElement(col_dummy_pivot,1,1)
	PokeFloat col_matrix, 24,GetMatElement(col_dummy_pivot,1,2)
		
	PokeFloat col_matrix, 32,GetMatElement(col_dummy_pivot,2,0)
	PokeFloat col_matrix, 36,GetMatElement(col_dummy_pivot,2,1)
	PokeFloat col_matrix, 40,GetMatElement(col_dummy_pivot,2,2)
	
	PokeFloat col_matrix, 48,EntityX(b_ent,1)
	PokeFloat col_matrix, 52,EntityY(b_ent,1)
	PokeFloat col_matrix, 56,EntityZ(b_ent,1)
	
	
coldet_set_matrix(c_ent,col_matrix)

End Function











;------------------------------------------------------------------------
;Get collision point for 'this' model.
;g=0 for global coordinates
;g=1 for model's local coordinates
;------------------------------------------------------------------------

Function coldet_collision_point(model,g)
	coldet_getcollision_point(model,col_point,g)
End Function

;and return the x,y,z values by these functions
Function collision_point_x#()
	Return PeekFloat (col_point,0)
End Function

Function collision_point_y#()
	Return PeekFloat (col_point,4)
End Function

Function collision_point_z#()
	Return PeekFloat (col_point,8)
End Function






;-------------------------------------------------------------------------------
;This function is actually used to get the colliding triangle normal.
;if calc1=true make precalculations for the first model's colliding tri normals
;if calc2=true make precalculations for the second model's colliding tri normals
;-------------------------------------------------------------------------------

Function coldet_colliding_verts(model,calc1,calc2)

	coldet_getcolliding_verts(model,col_verts1,col_verts2)


;v0x=0   , v0y=4   , v0z=8
;v1x=12  , v1y=16  , v1z=20
;v2x=24  , v2y=28  , v2z=32

If calc1=True
	cnrx1(0)=PeekFloat(col_verts1,12)-PeekFloat(col_verts1,0)
	cnry1(0)=PeekFloat(col_verts1,16)-PeekFloat(col_verts1,4)
	cnrz1(0)=PeekFloat(col_verts1,20)-PeekFloat(col_verts1,8)
	
	cnrx1(1)=PeekFloat(col_verts1,24)-PeekFloat(col_verts1,12)
	cnry1(1)=PeekFloat(col_verts1,28)-PeekFloat(col_verts1,16)
	cnrz1(1)=PeekFloat(col_verts1,32)-PeekFloat(col_verts1,20)
EndIf

If calc2=True
	cnrx2(0)=PeekFloat(col_verts2,12)-PeekFloat(col_verts2,0)
	cnry2(0)=PeekFloat(col_verts2,16)-PeekFloat(col_verts2,4)
	cnrz2(0)=PeekFloat(col_verts2,20)-PeekFloat(col_verts2,8)
	
	cnrx2(1)=PeekFloat(col_verts2,24)-PeekFloat(col_verts2,12)
	cnry2(1)=PeekFloat(col_verts2,28)-PeekFloat(col_verts2,16)
	cnrz2(1)=PeekFloat(col_verts2,32)-PeekFloat(col_verts2,20)
EndIf
	
End Function



Function colliding_nx1#()
	Return (cnry1(0)*cnrz1(1))-(cnrz1(0)*cnry1(1))
End Function

Function colliding_ny1#()
	Return (cnrz1(0)*cnrx1(1))-(cnrx1(0)*cnrz1(1))
End Function

Function colliding_nz1#()
	Return (cnrx1(0)*cnry1(1))-(cnry1(0)*cnrx1(1))
End Function




Function colliding_nx2#()
	Return (cnry2(0)*cnrz2(1))-(cnrz2(0)*cnry2(1))
End Function

Function colliding_ny2#()
	Return (cnrz2(0)*cnrx2(1))-(cnrx2(0)*cnrz2(1))
End Function

Function colliding_nz2#()
	Return (cnrx2(0)*cnry2(1))-(cnry2(0)*cnrx2(1))
End Function



;-----------------------------------------------------------------------------
;Get colliding triangle for 'this' model.
;If this=1 then the triangle of the 'other' colliding object will be returned.
;-----------------------------------------------------------------------------

Function coldet_colliding_tri(model,this=0)
	coldet_getcolliding_tris(model,col_tri1,col_tri2)
	
	If this=0
	Return PeekInt(col_tri1,0)
	Else
	Return PeekInt(col_tri2,0)
	EndIf
	
End Function